import java.io.*;
import java.util.Date;
import java.text.DateFormat;

/*
 * This class aids generic logging.  Right now you must log everything to one
 * file, but perhaps this class could be extended to also allow splitting the
 * logs across different files -- one for each datastream.  It could also be
 * extended to put datastreams directly into a relational database.
 *
 * In this context, a datastream is data coming from one particular object, or
 * one particular group of objects.  For example, stepsizes over time for a
 * particular area of the SPAR algorithm might constitute one datastream.
 */
public abstract class DataStream
{
  public static final int T_BOSS=0x001;
  public static final int T_PEON=0x002;
  public static final int T_VALU=0x004; // value function approximation
  public static final int T_STEP=0x00a; // stepsizes
  public static final int T_SIM =0x010; // overall simulation object,
  					// e.g. TwoAgentNewsvendor
  public static final int T_SYS =0x020; // general system information

  public static final int T_ALL =0xfff; // mask for all datastreams
  public static final int T_NONE=0x000; // mask for no datastreams

  int dataMask;

  /*
   * The dataMask is a hexadecimal mask composed of dataClass.  A set
   * dataClass implies that the corresponding datastream should be logged. 
   * 
   * Later we could move to a more generic facility in which you can add
   * datastreams, and each datastream that is added gets an identifier which
   * the owning code passes to the DataStream whenever the owning object writes
   * to the stream.  But right now its ok to compile in the identifiers.
   */

  /*
   * Logs a couple of nice comments.  Call at the end of the subclass'
   * constructor.
   */
  protected void PostInit()
  {
	  Date date = new Date();
	  PutComment(T_SYS, DateFormat.getDateTimeInstance(DateFormat.MEDIUM,
				  DateFormat.LONG).format(date));
	  PutComment(T_SYS, "DataMask = " + dataMask);
  }

  public abstract void PutComment(int dataClass, String s);
  public abstract void PutDataVector(int dataClass, double data[]);

  public void PutData(int dataClass, double ... data) {
	  PutDataVector(dataClass, data);
  }

  public void Cleanup() {}
};
