import java.lang.Math.*;
import java.lang.Boolean;

class GeometricTrendingStepsize extends OsaStepsize
{
  private double a;
  private double prevSample; // x_{n-1}
  private double prevDel; // x_{n-1} - x_{n-2}
  private Boolean notGeometric; 

  /*
   * In this implementation we use constant stepsize exponential smoothing to
   * collect trend information.  In the future we might use something more
   * sophisticated.
   */
  private double trendAlpha = 0.3;
  public GeometricTrendingStepsize(DataStream stream)
  {
	  super(stream);
	  a = 1; // initial estimate.
	  notGeometric = Boolean.FALSE;
  }

  // Think of sample as x_n
  public double Update(double sample)
  {
	  // If n==0 we don't have a previous sample and should just leave the
	  // smoothedTrend at 0.
	  /*
	   * We operate under the assumption that the dels, i.e. del_n =
	   * x_n - x_{n-1}, approximately follow the relation del_n =
	   * del_{n-1}*a, where a is some constant.  We estimate log a
	   * by smoothing log(del_n) - log(del_{n-1}).  Then we
	   * estimate del_{n+1} as del_n * a.
	   * 
	   * XXX what do we do about cases where del is <= 0?
	   * If a=0, then we could get such cases.
	   * Or if our noise is high, we could get such cases.
	   */

	  if (n == 0) {
		  prevSample = sample;
		  return super.Update(sample);
	  }

	  if (n == 1) {
		  prevDel = sample - prevSample;
		  prevSample = sample;
		  return super.Update(sample);
	  }

	  if (notGeometric)
		  return super.Update(sample);

	  // n >= 2
	  double del = sample - prevSample;
	  if (del <= 0 || prevDel <= 0) {
		  // XXX Right now this is coded so that hitting del<=0 once
		  // means a=0 forever after.  I think this can be handled
		  // better by a Kalman filter.
		  // XXX not right now
		  // notGeometric = Boolean.TRUE;
		  stream.PutComment(stream.T_STEP, "# Not Geometric");
		  a = 0;
	  } else {
		  double aSample = Math.exp(Math.log(del) - Math.log(prevDel));
		  a = (1-trendAlpha) * a + trendAlpha * aSample;
		  stream.PutComment(stream.T_STEP, "del=" + del +
				  " prevDel=" + prevDel + 
				  " aSample=" + aSample + 
				  " a=" + a);
	  }
	  // We should never see an "a" > 1.  This is a heuristic, and the
	  // theory should find a better way to do this.
	  if (a > 1)
		  a = 1;
	  thetabar += del * a; // does nothing if a is 0
	  prevSample = sample;
	  prevDel = del;
	  return super.Update(sample);
  }
}
