/*
 * This class takes a dataStream and averages.  This aids plotting because we
 * can take the average of many sample paths.  It also computes variances,
 * which tells you the quality of the sample averages.
 *
 * Note that this functionality could also be implemented by storing a
 * datastream into a relational database and then calling something like SELECT
 * AVG(COL1) FROM TABLE; with an appropriate GROUP BY.
 *
 */
public class MeanVarianceStream extends DataStream
{
  /*
   * Right now this code will only aggregate from one class from the
   * DataStream.  Later this might be extended.
   */
  int dataClass; 

  /*
   * Collect mean and variance for each data column at each iteration.
   */
  int iteration;
  int numCols; // number of columns in each iteration on the sample path
  int numRows; // number of iterations per sample path
  double x[][]; // current avergae of X at a particular row/col
  double x2[][]; // current average of X squared at a particular row/col
  int nSamples; // number of samples that goes into each x or x2 cell 

  /*
   * Right now you must specify numCols and numRows at initialization time.
   * Later we could allow this to grow dynamically, or to be set via some other
   * mechanism
   */
  public MeanVarianceStream(int dataClass, int numCols, int numRows)
  {
	  this.dataClass = dataClass;
	  this.numCols = numCols;
	  this.numRows = numRows;
	  x = new double[numRows][numCols];
	  x2 = new double[numRows][numCols];
	  for (int i=0; i < numRows; i++)
		  for (int j=0; j < numCols; j++) {
			  x[i][j] = 0;
			  x2[i][j] = 0;
		  }
	  iteration = 0;
	  nSamples = 0; // will be incremented to 1 by NewSamplePath();
  }

  // Discard all comments
  public void PutComment(int dataClass, String s) { }

  public void PutDataVector(int dataClass, double data[])
  {
	  if (dataClass != this.dataClass)
		  return;

	  if (iteration >= numRows) {
		  Warning("PutDataVector: iterations(" + iteration +
				  ") >= numRows (" + numRows + ")");
		  return;
	  }

	  // XXX Check that the length of the data vector is the same as
	  // numCols.  If not log an error.

	  for (int j = 0; j < numCols; j++) {
		  // The 1/n stepsize gives us a running average
		  // XXX Is this the way to do the average that gives us the
		  // best numerical precision?
		  double alpha = 1/(double)nSamples;
		  double newx = data[j];
		  double newx2 = newx * newx;
		  // XXX It might be faster to do these two updates in two
		  // separate loops because of fewer cache misses.
		  x[iteration][j] = (1-alpha)*x[iteration][j] + alpha*newx;
		  x2[iteration][j] = (1-alpha)*x2[iteration][j] + alpha*newx2;
	  }
	  iteration++;
  }

  /*
   * This is a special member function only in this particular extension of
   * DataStream.  It tells the StreamAverage object that the current data
   * stream has ended and that the stream will now repeat along a different
   * sample path.
   *
   * If each sample path does not have the same number of iterations then a
   * warning message will be logged.  Always call this right after
   * you call the constructor.
   */
  public void NewSamplePath() {
	  iteration = 0;
	  nSamples++;
  }

  /* 
   * Called by other member function to warns that the number of iterations is
   * different for a sample path.
   */ 
  private void Warning(String s) { System.err.println("[ERR] " + s); }

  /*
   * Tells the StreamAverage object that no more data is coming.
   * If the final sample path does not have the same number of iterations as
   * the other sample paths a warning message will be logged.
   */
  public void Cleanup() {
	  if (iteration != numRows)
		  Warning("Cleanup: iterations(" + iteration +
				  ") != numRows (" + numRows + ")");

	  for (iteration=0; iteration<numRows; iteration++) {
		  for (int j=0; j<numCols; j++) {
			  double mean = x[iteration][j];
			  double variance = x2[iteration][j] - mean*mean;
			  System.out.format("%.3f %.3f ", mean, variance);
		  }
		  System.out.format("%n");
	  }
	  System.out.println("#nSamples = " + nSamples);
  }
}
