/*
 * Abraham George's Optimal Stepsize Algorithm.
 * See his paper for a description of the algorithm.
 */
class OsaStepsize extends Stepsize
{
  protected int n; // iteration number
  protected double sigmahat, lambdabar, thetabar, deltabar, alpha, betabar, nu, nubar;
  DataStream stream;

  // XXX it would be very nice to allow, in addition to the stream,
  // the presence of an identifier that would distinguish this stepsize from
  // other instantiations of the stepsize.
  OsaStepsize(DataStream stream) { this(stream, 0, 1, 1, 0); }
  OsaStepsize(DataStream stream,
		  double thetabar0, double alpha1, double nu0, double nubar)
  {
	  this.stream = stream;
	  n = 0;
	  thetabar = thetabar0;
	  alpha = alpha1; // alpha will remain unchanged from step 0 to step 1. 
	  nu = nu0;
	  this.nubar = nubar;
	  deltabar = 0;
	  betabar = 0;
	  stream.PutComment(stream.T_STEP, "#n nu betabar deltabar sigmahat alpha lambdabar thetabar");
	  stream.PutData(stream.T_STEP, n,nu,betabar,deltabar, sigmahat, alpha, lambdabar, thetabar);
  }

  public double Update(double sample)
  {
	  /*
	   * When we start this routine, all private variables are from
	   * iteration n-1.
	   */

	  n++; // transforms n-1 to n (obviously).

	  nu = nu / (1 + nu - nubar); // transforms nu^{n-1} to nu^n

	  // transforms betabar^{n-1} to betabar^n
	  betabar = (1-nu) * betabar + nu*(thetabar - sample);

	  // transforms deltabar^{n-1} to deltabar^n
	  deltabar = (1-nu) * deltabar + nu*sq(thetabar - sample);

	  if (n > 1) {
		  // calculates sigmahat (really sigmahat^n squared)
		  sigmahat = (deltabar - sq(betabar))/(1 + lambdabar);

		  // calculates alpha^n
		  if (sigmahat == 0 && deltabar == 0)
			  alpha = 1; // 1-0/0 = 1
		  // XXX I should check for the case where deltabar == 0 and
		  // sigmabar != 0.  This should "never" happen, i.e. it should
		  // only happen if there is a bug in the code, or a bug in my
		  // understanding of the algorithm.  If it does happen, it
		  // would be nice to be alerted to it.
		  else
			  alpha = 1 - sigmahat/deltabar;

		  // transforms lambdabar^{n-1} to lambdabar^n
		  lambdabar = sq(1-alpha)*lambdabar + sq(alpha);
	  } else { // n == 1
		  // leave alpha alone
		  lambdabar = sq(alpha);
	  }

	  // transforms thetabar^{n-1} to thetabar^n
	  thetabar = (1-alpha)*thetabar + alpha*sample;

	  stream.PutData(n, nu, betabar, deltabar, sigmahat, alpha, lambdabar, thetabar);
	  return thetabar;
  }

  /* utility function for squaring */
  private double sq(double x) { return x*x; }

};
